@@ -136,7 +136,7 @@ def LoadFirmwareImage(chip: str, image_data: ImageSource):
         }[chip](f)
 
 
-class ImageSegment(object):
+class ImageSegment:
     """Wrapper class for a segment in an ESP image
     (very similar to a section in an ELFImage also)"""
 
@@ -170,9 +170,9 @@ class ImageSegment(object):
         return result
 
     def __repr__(self):
-        r = "len 0x%05x load 0x%08x" % (len(self.data), self.addr)
+        r = f"len {len(self.data):#07x} load {self.addr:#010x}"
         if self.file_offs is not None:
-            r += " file_offs 0x%08x" % (self.file_offs)
+            r += f" file_offs {self.file_offs:#010x}"
         return r
 
     def get_memory_type(self, image):
@@ -215,14 +215,14 @@ class ELFSection(ImageSegment):
     name as well as the common properties of an ImageSegment."""
 
     def __init__(self, name, addr, data, flags, align=4):
-        super(ELFSection, self).__init__(addr, data, flags=flags, align=align)
+        super().__init__(addr, data, flags=flags, align=align)
         self.name = name.decode("utf-8")
 
     def __repr__(self):
-        return "%s %s" % (self.name, super(ELFSection, self).__repr__())
+        return f"{self.name} {super().__repr__()}"
 
 
-class BaseFirmwareImage(object):
+class BaseFirmwareImage:
     SEG_HEADER_LEN = 8
     SHA256_DIGEST_LEN = 32
     IROM_ALIGN = 0
@@ -247,14 +247,14 @@ class BaseFirmwareImage(object):
         ) = struct.unpack("<BBBBI", load_file.read(8))
 
         if magic != expected_magic:
-            raise FatalError("Invalid firmware image magic=0x%x" % (magic))
+            raise FatalError(f"Invalid firmware image magic={magic:#x}")
         return segments
 
     def verify(self):
         if len(self.segments) > 16:
             raise FatalError(
-                "Invalid segment count %d (max 16). "
-                "Usually this indicates a linker script problem." % len(self.segments)
+                f"Invalid segment count {len(self.segments)} (max 16). "
+                "Usually this indicates a linker script problem."
             )
 
     def load_segment(self, f, is_irom_segment=False):
@@ -265,8 +265,8 @@ class BaseFirmwareImage(object):
         segment_data = f.read(size)
         if len(segment_data) < size:
             raise FatalError(
-                "End of file reading segment 0x%x, length %d (actual length %d)"
-                % (offset, size, len(segment_data))
+                f"End of file reading segment {offset:#x}, length {size} "
+                f"(actual length {len(segment_data)})"
             )
         segment = ImageSegment(offset, segment_data, file_offs)
         self.segments.append(segment)
@@ -298,8 +298,8 @@ class BaseFirmwareImage(object):
             ):
                 raise FatalError(
                     "Cannot place SHA256 digest on segment boundary"
-                    "(elf_sha256_offset=%d, file_pos=%d, segment_size=%d)"
-                    % (self.elf_sha256_offset, file_pos, segment_len)
+                    f"(elf_sha256_offset={self.elf_sha256_offset}, "
+                    f"file_pos={file_pos}, segment_size={segment_len})"
                 )
             # offset relative to the data part
             patch_offset -= self.SEG_HEADER_LEN
@@ -308,8 +308,9 @@ class BaseFirmwareImage(object):
                 != b"\x00" * self.SHA256_DIGEST_LEN
             ):
                 raise FatalError(
-                    "Contents of segment at SHA256 digest offset 0x%x are not all zero."
-                    " Refusing to overwrite." % self.elf_sha256_offset
+                    "Contents of segment at SHA256 digest offset "
+                    f"{self.elf_sha256_offset:#x} are not zero. "
+                    "Refusing to overwrite."
                 )
             assert len(self.elf_sha256) == self.SHA256_DIGEST_LEN
             segment_data = (
@@ -405,8 +406,8 @@ class BaseFirmwareImage(object):
         if len(irom_segments) > 0:
             if len(irom_segments) != 1:
                 raise FatalError(
-                    "Found %d segments that could be irom0. Bad ELF file?"
-                    % len(irom_segments)
+                    f"Found {len(irom_segments)} segments that could be irom0. "
+                    "Bad ELF file?"
                 )
             return irom_segments[0]
         return None
@@ -443,12 +444,9 @@ class BaseFirmwareImage(object):
                     elem_pad_addr != elem.addr + len(elem.data)
                     and elem_pad_addr == next_elem.addr
                 ):
-                    log.info(
-                        "Inserting {} bytes padding between {} and {}".format(
-                            next_elem.addr - (elem.addr + len(elem.data)),
-                            elem.name,
-                            next_elem.name,
-                        )
+                    log.note(
+                        f"Inserting {next_elem.addr - (elem.addr + len(elem.data))} "
+                        f"bytes padding between {elem.name} and {next_elem.name}"
                     )
                     elem.pad_until_addr(elem_pad_addr)
             if all(
@@ -511,7 +509,7 @@ class ESP8266ROMFirmwareImage(BaseFirmwareImage):
     ROM_LOADER = ESP8266ROM
 
     def __init__(self, load_file=None):
-        super(ESP8266ROMFirmwareImage, self).__init__()
+        super().__init__()
         self.flash_mode = 0
         self.flash_size_freq = 0
         self.version = 1
@@ -581,7 +579,7 @@ class ESP8266V2FirmwareImage(BaseFirmwareImage):
     IMAGE_V2_SEGMENT = 4
 
     def __init__(self, load_file=None):
-        super(ESP8266V2FirmwareImage, self).__init__()
+        super().__init__()
         self.version = 2
         if load_file is not None:
             segments = self.load_common_header(load_file, self.IMAGE_V2_MAGIC)
@@ -641,10 +639,8 @@ class ESP8266V2FirmwareImage(BaseFirmwareImage):
             irom_offs = irom_segment.addr - ESP8266ROM.IROM_MAP_START
         else:
             irom_offs = 0
-        return "%s-0x%05x.bin" % (
-            os.path.splitext(input_file)[0],
-            irom_offs & ~(ESPLoader.FLASH_SECTOR_SIZE - 1),
-        )
+        sector_mask = ~(ESPLoader.FLASH_SECTOR_SIZE - 1)
+        return f"{os.path.splitext(input_file)[0]}-{irom_offs & sector_mask:#07x}.bin"
 
     def save(self, filename: str | None) -> bytes | None:
         with io.BytesIO() as f:  # Write to memory first
@@ -710,7 +706,7 @@ class ESP32FirmwareImage(BaseFirmwareImage):
     can be placed in the normal image (just @ MMU page size padded offsets).
     """
 
-    ROM_LOADER = ESP32ROM
+    ROM_LOADER: type[ESPLoader] = ESP32ROM
 
     # ROM bootloader will read the wp_pin field if SPI flash
     # pins are remapped via flash. IDF actually enables QIO only
@@ -723,7 +719,7 @@ class ESP32FirmwareImage(BaseFirmwareImage):
     IROM_ALIGN = 65536
 
     def __init__(self, load_file=None, append_digest=True, ram_only_header=False):
-        super(ESP32FirmwareImage, self).__init__()
+        super().__init__()
         self.secure_pad = None
         self.flash_mode = 0
         self.flash_size_freq = 0
@@ -773,7 +769,7 @@ class ESP32FirmwareImage(BaseFirmwareImage):
 
     def default_output_name(self, input_file):
         """Derive a default output name from the ELF name."""
-        return "%s.bin" % (os.path.splitext(input_file)[0])
+        return f"{os.path.splitext(input_file)[0]}.bin"
 
     def warn_if_unusual_segment(self, offset, size, is_irom_segment):
         pass  # TODO: add warnings for wrong ESP32 segment offset/size combinations
@@ -1211,7 +1207,7 @@ ESP32C2ROM.BOOTLOADER_IMAGE = ESP32C2FirmwareImage
 class ESP32C6FirmwareImage(ESP32FirmwareImage):
     """ESP32C6 Firmware Image almost exactly the same as ESP32FirmwareImage"""
 
-    ROM_LOADER = ESP32C6ROM
+    ROM_LOADER: type[ESPLoader] = ESP32C6ROM
     MMU_PAGE_SIZE_CONF = (8192, 16384, 32768, 65536)
 
 
@@ -1236,19 +1232,11 @@ class ESP32C5FirmwareImage(ESP32FirmwareImage):
 ESP32C5ROM.BOOTLOADER_IMAGE = ESP32C5FirmwareImage
 
 
-class ESP32H4FirmwareImage(ESP32FirmwareImage):
-    """ESP32H4 Firmware Image almost exactly the same as ESP32FirmwareImage"""
+class ESP32H4FirmwareImage(ESP32C6FirmwareImage):
+    """ESP32H4 Firmware Image almost exactly the same as ESP32C6FirmwareImage"""
 
     ROM_LOADER = ESP32H4ROM
 
-    def set_mmu_page_size(self, size):
-        if size not in [8192, 16384, 32768, 65536]:
-            raise FatalError(
-                "{} bytes is not a valid ESP32-H4 page size, "
-                "select from 64KB, 32KB, 16KB, 8KB.".format(size)
-            )
-        self.IROM_ALIGN = size
-
 
 ESP32H4ROM.BOOTLOADER_IMAGE = ESP32H4FirmwareImage
 
@@ -1280,15 +1268,7 @@ class ESP32H21FirmwareImage(ESP32C6FirmwareImage):
 ESP32H21ROM.BOOTLOADER_IMAGE = ESP32H21FirmwareImage
 
 
-class ELFFile(object):
-    SEC_TYPE_PROGBITS = 0x01
-    SEC_TYPE_STRTAB = 0x03
-    SEC_TYPE_NOBITS = 0x08  # e.g. .bss section
-    SEC_TYPE_INITARRAY = 0x0E
-    SEC_TYPE_FINIARRAY = 0x0F
-
-    PROG_SEC_TYPES = (SEC_TYPE_PROGBITS, SEC_TYPE_INITARRAY, SEC_TYPE_FINIARRAY)
-
+class ELFFile:
     LEN_SEC_HEADER = 0x28
 
     SEG_TYPE_LOAD = 0x01
@@ -1303,7 +1283,7 @@ class ELFFile(object):
         for s in self.sections:
             if s.name == section_name:
                 return s
-        raise ValueError("No section %s in ELF file" % section_name)
+        raise ValueError(f"No section {section_name} in ELF file")
 
     def _read_elf_file(self, f):
         # read the ELF file header
@@ -1347,6 +1327,22 @@ class ELFFile(object):
         self._read_segments(f, _phoff, _phnum, shstrndx)
 
     def _read_sections(self, f, section_header_offs, section_header_count, shstrndx):
+        SEC_TYPE_PROGBITS = 0x01
+        SEC_TYPE_STRTAB = 0x03
+        SEC_TYPE_NOBITS = 0x08  # e.g. .bss section
+        SEC_TYPE_INITARRAY = 0x0E
+        SEC_TYPE_FINIARRAY = 0x0F
+        SEC_TYPE_PREINITARRAY = 0x10
+
+        PROG_SEC_TYPES = (
+            SEC_TYPE_PROGBITS,
+            SEC_TYPE_INITARRAY,
+            SEC_TYPE_FINIARRAY,
+            SEC_TYPE_PREINITARRAY,
+        )
+
+        KNOWN_SEC_TYPES = PROG_SEC_TYPES + (SEC_TYPE_NOBITS, SEC_TYPE_STRTAB)
+
         f.seek(section_header_offs)
         len_bytes = section_header_count * self.LEN_SEC_HEADER
         section_header = f.read(len_bytes)
@@ -1378,17 +1374,13 @@ class ELFFile(object):
             ) = struct.unpack_from("<LLLLLLLLL", section_header[offs:])
             return (name_offs, sec_type, lma, size, sec_offs, _flags, align)
 
-        all_sections = [read_section_header(offs) for offs in section_header_offsets]
-        prog_sections = [s for s in all_sections if s[1] in ELFFile.PROG_SEC_TYPES]
-        nobits_secitons = [s for s in all_sections if s[1] == ELFFile.SEC_TYPE_NOBITS]
-
         # search for the string table section
         if (shstrndx * self.LEN_SEC_HEADER) not in section_header_offsets:
             raise FatalError(f"ELF file has no STRTAB section at shstrndx {shstrndx}")
         _, sec_type, _, sec_size, sec_offs, _flags, align = read_section_header(
             shstrndx * self.LEN_SEC_HEADER
         )
-        if sec_type != ELFFile.SEC_TYPE_STRTAB:
+        if sec_type != SEC_TYPE_STRTAB:
             log.warning(f"ELF file has incorrect STRTAB section type {sec_type:#04x}")
         f.seek(sec_offs)
         string_table = f.read(sec_size)
@@ -1404,25 +1396,46 @@ class ELFFile(object):
             f.seek(offs)
             return f.read(size)
 
-        prog_sections = [
-            ELFSection(
-                lookup_string(n_offs),
-                lma,
-                read_data(offs, size),
-                flags=_flags,
-                align=align,
-            )
-            for (n_offs, _type, lma, size, offs, _flags, align) in prog_sections
-            if lma != 0 and size > 0
-        ]
-        self.sections = prog_sections
-        self.nobits_sections = [
-            ELFSection(lookup_string(n_offs), lma, b"", flags=_flags, align=align)
-            for (n_offs, _type, lma, size, offs, _flags, align) in nobits_secitons
-            if lma != 0 and size > 0
-        ]
+        all_sections = [read_section_header(offs) for offs in section_header_offsets]
+
+        self.sections = []
+        self.nobits_sections = []
+        # Process all sections and raise an error if an unknown section type is found
+        for section in all_sections:
+            n_offs, sec_type, lma, size, offs, _flags, align = section
+
+            # Skip sections with lma == 0 or size == 0
+            if lma == 0 or size == 0:
+                continue
+
+            if sec_type not in KNOWN_SEC_TYPES:
+                log.warning(f"Unknown section type {sec_type:#04x} in ELF file")
+                continue
+
+            if sec_type in PROG_SEC_TYPES:
+                self.sections.append(
+                    ELFSection(
+                        lookup_string(n_offs),
+                        lma,
+                        read_data(offs, size),
+                        flags=_flags,
+                        align=align,
+                    )
+                )
+            elif sec_type == SEC_TYPE_NOBITS:
+                self.nobits_sections.append(
+                    ELFSection(
+                        lookup_string(n_offs),
+                        lma,
+                        b"",
+                        flags=_flags,
+                        align=align,
+                    )
+                )
 
     def _read_segments(self, f, segment_header_offs, segment_header_count, shstrndx):
+        if segment_header_count == 0:
+            return
         f.seek(segment_header_offs)
         len_bytes = segment_header_count * self.LEN_SEG_HEADER
         segment_header = f.read(len_bytes)
